rm (list = ls())

library(timelineS)
library(lubridate)
library(ggplot2)
library(dplyr)
library(reshape2)
library(rPref)
library(mco)
library(MASS)

plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) +
                        years(lifetime))
  timelineS(dataP, main = plot.Name,
            labels = events.name, label.direction = "up", label.position = 3)
}

dist.Events <- function (lifetime, events, start.Date, option.Name) {
  #sort the events
  events <- sort(events, decreasing = T)
  #create the distribution of the events
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  #all events unlisted
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  #sort the events ascending
  all.events <- all.events[order(all.events$frequency),]
  #get the unique sequence of events
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  #plot the timeline
  #plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}

combine.lifeTimelines <- function(product.1, product.2,product.3, p1.dur, p2.dur, p3.dur) {
  
  base <- list(Names = c(), frequency = c(), duration = c())
  base$frequency <- sort(unique(c(product.1$frequency,
                                  product.2$frequency,
                                  product.3$frequency)),
                         decreasing = FALSE)
  
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  
  for(index in 2:(length(base$frequency) - 1)) {        
    phase.1 <- product.1$event[which(product.1$frequency ==
                                       base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency ==
                                       base$frequency[index])]
    phase.3 <- product.3$event[which(product.3$frequency ==
                                       base$frequency[index])]
    
    if(length(phase.1) != 0 && length(phase.2) != 0 && length(phase.3) != 0) {
      base$Names[index] <- paste(phase.1, phase.2, phase.3)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)],
                                  p3.dur[which(names(p3.dur) == phase.3)])
      
    } else if(length(phase.1) != 0 && length(phase.2) == 0 && length(phase.3) != 0) {
      base$Names[index] <- paste(phase.1, phase.3)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p3.dur[which(names(p3.dur) == phase.3)])
      
    }  else if(length(phase.1) != 0 && length(phase.2) == 0 && length(phase.3) == 0) {
      base$Names[index] <- phase.1
      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
      
    }else if(length(phase.1) != 0 && length(phase.2) != 0 && length(phase.3) == 0) {
      base$Names[index] <- paste(phase.1, phase.2)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)]) 
      
    }else if(length(phase.1) == 0 && length(phase.2) != 0 && length(phase.3) == 0) {
      base$Names[index] <- phase.2
      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
      
    } else if(length(phase.1) == 0 && length(phase.2) != 0 && length(phase.3) != 0) {
      base$Names[index] <- paste(phase.3, phase.2)
      base$duration[index] <- max(p3.dur[which(names(p3.dur) == phase.3)],
                                  p2.dur[which(names(p2.dur) == phase.2)]) 
      
    }else if(length(phase.1) == 0 && length(phase.2) == 0 && length(phase.3) != 0) {
      base$Names[index] <- phase.3
      base$duration[index] <- p3.dur[which(names(p3.dur) == phase.3)]
      
    } else {
      base$duration[index] <- 0
    }
  }
  return(base)
}

par(mfrow = c(2, 1))
start.Date = "2023-01-01"
lifetime <- 60 

events.Op2.Dam = c(LM = 15, M = 10, CM = 20, END = lifetime)
duration.ev.Dam <- c(LM = 4, M = 2, CM = 15)

events.Op1.Pavement <- c(AO.p = 12, M.p = 5, PR.p = 32, END = lifetime)
duration.ev.Pavement <- c(AO.p = 7, M.p = 2, PR.p = 20)

events.Op1.highwayBridge <- c(SDO.b = 15, M.b = 5, DR.b = 32, END = lifetime)
duration.ev.highwayBridge <- c(SDO.b = 7, M.b = 2, DR.b = 20)

#design.Options.Dam <- list(desing.Op2 <- "2.Rock-fill dam with an asphaltic concrete core")
#design.Options.Pavement <- list(desing.Op1.Pavement<- "1. Cement-treated base pavement with HMA surface")
#design.Options.highwayBridge <- list(desing.Op1.highwayBridge <- "1.Highway Brdige with reinforced concrete pier and Cast in place pier cap")

design.Options.Dam <- list(desing.Op2.Dam <- "2.Rock-fill dam with an asphaltic concrete core")
maintenance.Dam <- dist.Events(lifetime, events.Op2.Dam, start.Date, design.Options.Dam)

design.Options.Pavement <- list(desing.Op1.Pavement<- "1. Cement-treated base pavement with HMA surface")
maintenance.Pavement <- dist.Events(lifetime, events.Op1.Pavement, start.Date, design.Options.Pavement)

design.Options.highwayBridge <- list(desing.Op1.highwayBridge<- "1. Highway Bridge with reinforced concrete pier and Cast in place pier cap")
maintenance.highwayBridge <- dist.Events(lifetime, events.Op1.highwayBridge, start.Date, design.Options.highwayBridge)

integrated.interv <- combine.lifeTimelines(maintenance.Dam, maintenance.Pavement,maintenance.highwayBridge,
                                           duration.ev.Dam, duration.ev.Pavement, duration.ev.highwayBridge)


sum(integrated.interv$duration)


events.Op2.Dam = c(LM=15, M = 10, CM = 20, END = lifetime)
duration.ev.Dam <- c(LM=4, M = 2, CM = 15)
events.Op1.Pavement <- c(AO.p = 12, M.p = 5, PR.p = 32, END = lifetime)
duration.ev.Pavement <- c(AO.p = 7, M.p = 2, PR.p = 20)
events.Op1.highwayBridge <- c(SDO.b = 15, M.b = 5, DR.b = 32, END = lifetime)
duration.ev.highwayBridge <- c(SDO.b = 7, M.b = 2, DR.b = 20)

 design.Options.Dam <- list(desing.Op2 <- "2.Rock-fill dam with an asphaltic concrete core")
 design.Options.Pavement <- list(desing.Op1.Pavement<- "1. Cement-treated base pavement with HMA surface")
design.Options.highwayBridge <- list(desing.Op1.highwayBridge <- "1.Highway Bridge with reinforced concrete pier and Cast in place pier cap")

maintenance.Dam <- dist.Events(lifetime, events.Op2.Dam, start.Date, design.Options.Dam)
maintenance.Pavement <- dist.Events(lifetime, events.Op1.Pavement, start.Date, design.Options.Pavement)
maintenance.highwayBridge <- dist.Events(lifetime, events.Op1.highwayBridge, start.Date, design.Options.highwayBridge)

integrated.interv <- combine.lifeTimelines(maintenance.Dam, maintenance.Pavement,maintenance.highwayBridge,
                                           duration.ev.Dam, duration.ev.Pavement,duration.ev.highwayBridge
                                           )

sum(integrated.interv$duration)


design.explore <- function(events1, events2, events3) {
  results <- c()
  for(i in 1: dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, design.Options.Dam)
    dur.ev1 <- ev1/3
    for (j in 1: dim(events1)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Options.Pavement)
      dur.ev2 <- ev2/3
      for (k in 1: dim(events1)[1]) {
        ev3 <- unlist(events3[k, ])
        dist.3 <- dist.Events(lifetime, ev3, start.Date, design.Options.highwayBridge)
        dur.ev3 <- ev3/3 
        combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dist.3,
                                                 dur.ev1, dur.ev2, dur.ev3)
        min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] -
                                combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
        results <- rbind(results, c(ev1, ev2, ev3, dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
      }
    }
  }
  return(as.data.frame(results))
}

n.grid <- 3

events.grid.Dam <- expand.grid( LM = sample(seq(12,18,by = 1), n.grid),
                                  M = sample(seq(5,11,by = 1), n.grid),
                                  CM = sample(seq(17, 24, 1), n.grid))

events.grid.Pavement <- expand.grid(AO.p = sample(seq(8, 15, by = 1), n.grid),
                                M.p = sample(seq(3,8,1), n.grid),
                                PR.p = sample(seq(28, 35, 1), n.grid))

events.grid.highwayBridge <- expand.grid(SDO.b = sample(seq(12, 18, by = 1), n.grid),
                                    M.b = sample(seq(3,8,1), n.grid),
                                    PR.b = sample(seq(28, 35, 1), n.grid))


response.space <- design.explore(events.grid.Dam, events.grid.Pavement, events.grid.highwayBridge)

p <- low(dur)* high(dist.inter)

sky <- psel(response.space, p)

pareto2 <- psel(response.space, p, top = nrow(response.space))

ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))


show_front <- function(pref) {
  plot(response.space$dur, response.space$dist.inter)
  sky <- psel(response.space, pref)
  plot_front(response.space, pref, col = rgb(0, 0, 1))
  points(sky$dur, sky$dist.inter, lwd = 3)
}

show_front(p)

p <- high(dur) * low(dist.inter)
show_front(p)

LCI.materials <- read.csv("LCImaterialsALL.csv")
LCA.System <- function(length, width, height, thickness, hma.Option, girder.Option, system.Option, materials, dist.event) {

  prefab.girder.Section <- 0.78

  #based on the use of each material, we will split the data frame
  materials.split <- split(materials, materials$scope)

  #new line to get the interventions
  s <- summary(as.factor(dist.event$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))

  # system options
  if(system.Option == "CTB") {
    system.volume <- length * width * height
    system.interventions <- s2$DC + s2$PR.p + 0.3 * (if(!is.null(s2$M.p)) {s2$M.p} else {0}) + 
      0.55 * (if(!is.null(s2$AO.p)) {s2$AO.p} else {0})


  } else if (system.Option == "ACC") {
    system.volume <- 0.15 * length * width * height
    system.interventions <- s2$DC + 0.15 * s2$M + 0.4 * (if(!is.null(s2$LM)) {s2$LM} else {0}) + 
      0.65 * (if(!is.null(s2$CM)) {s2$CM} else {0})


  } else if (system.Option == "RC") {
    system.volume <- length * width * height
    system.interventions <- s2$DC + s2$DR.b + 0.25 * s2$SDO.b

  }

  #girder options
  if (girder.Option == "PRC") {
    #get the numbers of girders
    n <-round(width / 3.75, 0)
    interventions.girders <- s2$DC + s2$DR.b
    #get the volume of the concrete for the prefab girders
    girders.V <- n * prefab.girder.Section * length
  } else if (girder.Option == "none") {
    n <- 0
    girders.V <- 0
    interventions.girders <- 0
  }
  #hma options
  if (hma.Option == "hma") {
    hma.Q <- length * width * thickness
    hma.interventions <- length(dist.event$event)-1
  } else if (hma.Option == "none") {
    hma.Q <- 0
    hma.interventions <- 0
  }

  system <- mutate(materials.split[[system.Option]], system.Q = system.volume, interventions =
                     system.interventions)

  if (!is.null(materials.split[[girder.Option]]) && !is.null(materials.split[[hma.Option]])) {
    girders <- mutate(materials.split[[girder.Option]], system.Q = girders.V, interventions =
                        interventions.girders)
    hma <- mutate(materials.split[[hma.Option]], system.Q = hma.Q, interventions = hma.interventions)
    LCA.matrix <- rbind(system, girders, hma)
  } else if (is.null(materials.split[[girder.Option]]) && !is.null(materials.split[[hma.Option]])){
    hma <- mutate(materials.split[[hma.Option]], system.Q = hma.Q, interventions = hma.interventions)
    LCA.matrix <- rbind(system, hma)
  } else if (!is.null(materials.split[[girder.Option]]) && is.null(materials.split[[hma.Option]])){
    girders <- mutate(materials.split[[girder.Option]], system.Q = girders.V, interventions = interventions.girders)
    LCA.matrix <- rbind(system, girders)
  } else {
    LCA.matrix <- rbind(system)
  }

  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * system.Q/ 1000,
                       materials.LC = TotalMaterials.Q* interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOx.LC = materials.LC * NOx * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))

  return(LCA.results)

}

b.length <- 16 # units: m
b.width <- 15 #units m
bd.depth <- 0.25 #units m
asphalt.tk <- 0.12 #units m
p.length <- 1.00 # units: m
p.width <- 1.00 #units m
p.depth <- 0.20 #units m
hma.tk <- 0.175 #units m
d.length <- 350 # units: m
d.width <- 5.25 #units m
d.height <- 8.25 #units m

girder.Options <- c("PRC", "none")
hma.options <- c("hma","none")
system.options<- c("CTB", "ACC", "RC")

Option.bridge <- LCA.System(b.length, b.width, bd.depth, asphalt.tk, hma.options[2],
                            girder.Options[1], system.options[3], LCI.materials, maintenance.highwayBridge)
Option.Pavement <- LCA.System(p.length, p.width, p.depth, hma.tk, hma.options[1],
                              girder.Options[2], system.options[1], LCI.materials, maintenance.Pavement)
Option.Dam <- LCA.System(d.length, d.width, d.height, thickness = 0, hma.options[2],
                         girder.Options[2], system.options[2], LCI.materials, maintenance.Dam)

integrated.Design <- as.data.frame(list(Energy = Option.bridge$Energy + Option.Pavement$Energy+ Option.Dam$Energy,
                                        CO2 = Option.bridge$CO2 + Option.Pavement$CO2+ Option.Dam$CO2,
                                        NOx = Option.bridge$NOx + Option.Pavement$NOx+ Option.Dam$NOx,
                                        SO2 = Option.bridge$SO2 + Option.Pavement$SO2+ Option.Dam$SO2))


Energy.costs <- 0.128
CO2.unitcost <- 26 # per metric tone
NOx.unitCost <- 42 # per metric tone
SO2.unitCosts <- 85 # per metric tone
integrated.Design <- mutate(integrated.Design,
                            Costs = (Energy * Energy.costs +
                                       CO2*CO2.unitcost +
                                       NOx * NOx.unitCost +
                                       SO2 * SO2.unitCosts)/10^9)

fitness <- function(x) {
  #define the output dimension
  z <- numeric(7)
  x <- round(x, 0)
  y <- expand.grid(LM = x[1], M = x[2], CM = x[3], 
                   AO.p = x[4], M.p = x[5], PR.p = x[6],
                   SDO.b = x[7], M.b = x[8], DR.b = x[9])
  dur.ev1 <- unlist(y[1:3] / 3)
  dur.ev2 <- unlist(y[4:6] / 3)
  dur.ev3 <- unlist(y[7:9] / 3)
  dist.1 <- apply(y[1:3], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.2 <- apply(y[4:6], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.3 <- apply(y[7:9], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dist.3[[1]],
                                   dur.ev1, dur.ev2, dur.ev3)
  z[1] <- sum(results[["duration"]])
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] -
                     results$frequency[2:length(results$frequency)]))
  LCI.materials <- read.csv("LCImaterialsALL.csv")
  b.length <- 16 # units: m
  b.width <- 15 #units m
  bd.depth <- 0.25 #units m
  asphalt.tk <- 0.12 #units m
  p.length <- 1.00 # units: m
  p.width <- 1.00 #units m
  p.depth <- 0.20 #units m
  hma.tk <- 0.175 #units m
  d.length <- 350 # units: m
  d.width <- 5.25 #units m
  d.height <- 8.25 #units m
  Energy.costs <- 0.128
  CO2.unitcost <- 26 # per metric tone
  NOx.unitCost <- 42 # per metric tone
  SO2.unitCosts <- 85 # per metric tone
  girder.Options <- c("PRC", "none")
  hma.options <- c("hma","none")
  system.options<- c("CTB", "ACC", "RC")
  Option.bridge <- LCA.System(b.length, b.width, bd.depth, asphalt.tk, hma.options[2],
                              girder.Options[1], system.options[3], LCI.materials, dist.3[[1]])
  Option.Pavement <- LCA.System(p.length, p.width, p.depth, hma.tk, hma.options[1],
                                girder.Options[2], system.options[1], LCI.materials, dist.2[[1]])
  
  Option.Dam <- LCA.System(d.length, d.width, d.height, thickness = 0, hma.options[2],
                           girder.Options[2], system.options[2], LCI.materials, dist.1[[1]])
  integrated.system <- as.data.frame(list(Energy = Option.bridge$Energy + Option.Pavement$Energy+ Option.Dam$Energy,
                                          CO2 = Option.bridge$CO2 + Option.Pavement$CO2+ Option.Dam$CO2,
                                          NOx = Option.bridge$NOx + Option.Pavement$NOx+ Option.Dam$NOx,
                                          SO2 = Option.bridge$SO2 + Option.Pavement$SO2+ Option.Dam$SO2))
  integrated.system <- mutate(integrated.system,
                              Costs = (Energy * Energy.costs +
                                         CO2*CO2.unitcost +
                                         NOx * NOx.unitCost +
                                         SO2 * SO2.unitCosts)/10^9)
  z[3] <- integrated.system$Energy
  z[4] <- integrated.system$CO2
  z[5] <- integrated.system$NOx
  z[6] <- integrated.system$SO2
  z[7] <- integrated.system$Costs
  return(z)
}

r2 <- nsga2(fitness, idim = 9, odim = 7,
            generations= 10, popsize=100,
            lower.bounds=c(12, 5, 17, 8, 3, 28, 12, 3, 28),
            upper.bounds=c(18, 11, 24, 15, 8, 35, 18, 8, 35))

r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" )
colnames(r2Results) <- outNames
pareto3 <- as.data.frame(paretoFront(r2))
colnames(pareto3) <- outNames
ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")
input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c("LM", "M", "CM",
                           "AO.p", "M.p", "PR.p",
                           "SDO.b", "M.b", "DR.b", 
                           "duration", "interv.dist",
                           "energy", "co2", "nox",
                           "so2", "cost", "pareto")
par(mfrow = c(1,1))
parcoord(all.results[, 1:16], var.label = T,
         col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"),
         lty = ifelse(all.results$pareto == TRUE, 1, 3),
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))







